@@ -1,7 +1,7 @@
 import torch 
 import torch.nn as nn 
 import torch.nn.functional as F
-
+import random
 from bert_seq2seq.tokenizer import Tokenizer, load_chinese_base_vocab
 import time
 from bert_seq2seq.config import yayun_list
@@ -78,6 +78,7 @@ class Seq2SeqModel(BasicBert):
             return predictions, loss 
         else :
             return predictions
+
     
     def generate(self, text, out_max_length=40, beam_size=1, is_poem=False, max_length=256):
         # 对 一个 句子生成相应的结果
@@ -94,12 +95,78 @@ class Seq2SeqModel(BasicBert):
         else :   
             out_puts_ids = self.beam_search(token_ids, token_type_ids, self.word2ix, beam_size=beam_size, device=self.device)
         
-        # 解码 得到相应输出
-        # if err is False:
-        #     return self.tokenizer.decode(out_puts_ids)
-        
         return self.tokenizer.decode(out_puts_ids.cpu().numpy())
     
+    def generate_random(self, text, out_max_length=40, beam_size=3, max_length=256):
+        # 对 一个 句子生成相应的结果
+        ## 通过输出最大长度得到输入的最大长度，这里问题不大，如果超过最大长度会进行截断
+        self.out_max_length = out_max_length
+        input_max_length = max_length - out_max_length
+        token_ids, token_type_ids = self.tokenizer.encode(text, max_length=input_max_length)
+        token_ids = torch.tensor(token_ids, device=self.device).view(1, -1)
+        token_type_ids = torch.tensor(token_type_ids, device=self.device).view(1, -1)
+         
+        out_puts_ids_list = self.beam_search_list(token_ids, token_type_ids, self.word2ix, beam_size=beam_size, device=self.device)
+        random_int = random.randint(0, len(out_puts_ids_list) - 1)
+        return self.tokenizer.decode(out_puts_ids_list[random_int].cpu().numpy())
+
+    def beam_search_list(self, token_ids, token_type_ids, word2ix, beam_size=3, device="cpu"):
+        """
+        beam-search操作
+        """
+        sep_id = word2ix["[SEP]"]
+        output_list = []
+        # 用来保存输出序列
+        output_ids = torch.empty(1, 0, device=device, dtype=torch.long)
+        # 用来保存累计得分
+        with torch.no_grad(): 
+            output_scores = torch.zeros(token_ids.shape[0], device=device)
+            for step in range(self.out_max_length):
+                if step == 0:
+                    scores = self.forward(token_ids, token_type_ids)
+                    # 重复beam-size次 输入ids
+                    token_ids = token_ids.view(1, -1).repeat(beam_size, 1)
+                    token_type_ids = token_type_ids.view(1, -1).repeat(beam_size, 1)
+                else:
+                    scores = self.forward(new_input_ids, new_token_type_ids)
+                
+                logit_score = torch.log_softmax(scores[:, -1], dim=-1)
+                
+                logit_score = output_scores.view(-1, 1) + logit_score # 累计得分
+                ## 取topk的时候我们是展平了然后再去调用topk函数
+                # 展平
+                logit_score = logit_score.view(-1)
+                hype_score, hype_pos = torch.topk(logit_score, beam_size)
+                indice1 = (hype_pos // scores.shape[-1]) # 行索引
+                indice2 = (hype_pos % scores.shape[-1]).long().reshape(-1, 1) # 列索引
+               
+                # 更新得分
+                output_scores = hype_score
+                output_ids = torch.cat([output_ids[indice1], indice2], dim=1).long()
+                new_input_ids = torch.cat([token_ids, output_ids], dim=1)
+                new_token_type_ids = torch.cat([token_type_ids, torch.ones_like(output_ids)], dim=1)
+
+                end_counts = (output_ids == sep_id).sum(1)  # 统计出现的end标记
+               
+                flag = (end_counts < 1)  # 标记未完成序列
+                if not flag.all():  # 如果有已完成的
+                    for i, f in enumerate(flag):
+                        output_list.append(output_ids[i][:-1])
+                    token_ids = token_ids[flag]
+                    token_type_ids = token_type_ids[flag]
+                    new_input_ids = new_input_ids[flag]
+                    new_token_type_ids = new_token_type_ids[flag]
+                    output_ids = output_ids[flag]  # 扔掉已完成序列
+                    output_scores = output_scores[flag]  # 扔掉已完成序列
+                    end_counts = end_counts[flag]  # 扔掉已完成end计数
+                    beam_size = flag.sum()  # topk相应变化
+
+                if beam_size < 1:
+                    # print(output_list)
+                    return output_list
+            # print(output_list)
+            return output_list
+    
     # def poem_beam_search(self, token_ids, token_type_ids, word2ix, beam_size=1, device="cpu"):
     #     """
     #     专门针对写诗的beam-search
